@@ -1,24 +1,14 @@
-from gql import gql
-from .nodes import (
-    Session,
-    Image,
-    imageFragment,
-    ProgramInvitation,
-    ProgramTrackActionResponse,
-    programTrackActionResponseFragment,
-    ProgramTrackAction,
-    programTrackActionFragment,
-    ProgramModule,
-    programModuleFragment,
-    Tag,
-    tagFragment,
-)
+from .nodes import *
 from .context import Context
-from typing import Optional, Dict, Any
+from typing import Optional, Dict, Any, List
 import datetime
 import asyncio
-import traceback
-from ..gql import GqlClient
+import time
+from ..gql import GqlClient, gql
+import logging
+import httpx
+
+log = logging.getLogger(__name__)
 
 
 class ObschartApiClient(object):
@@ -33,10 +23,10 @@ class ObschartApiClient(object):
         self._gql_client = gql_client
         self._context = Context(self)
 
-    def _execute(self, query, variables: Optional[Dict[str, Any]] = None):
-        return self._gql_client.execute(query, variables)
+    def _execute(self, query, variables: Optional[Dict[str, Any]] = None, *, timeout=None):
+        return self._gql_client.execute(query, variables, timeout=timeout)
 
-    def _execute_mutation(self, query, input: Optional[Dict[str, Any]] = None):
+    def _execute_mutation(self, query, input: Optional[Dict[str, Any]] = None, *, timeout=None):
         variables = {"input": input}
         return self._execute(query, variables)
 
@@ -86,18 +76,22 @@ class ObschartApiClient(object):
         variables = {"filter": filter}
 
         while True:
+            start = time.monotonic()
             try:
-                response = await self._execute(query, variables)
+                response = await self._execute(query, variables, timeout=1)
                 edges = response["programTrackActionResponses"]["edges"]
 
                 if len(edges) > 0:
                     break
-            except Exception as exception:
-                traceback.print_exception(None, exception, exception.__traceback__)
+            except httpx.TimeoutException as exception:
+                log.warn(f"Polling timed out: {exception.__class__.__name__}")
+            except:
+                log.exception("Polling failed")
 
-            await asyncio.sleep(1)
+            duration = time.monotonic() - start
+            await asyncio.sleep(max(1 - duration, 0))
 
-        responses: list[ProgramTrackActionResponse] = [
+        responses: List[ProgramTrackActionResponse] = [
             ProgramTrackActionResponse(edge["node"], self._context) for edge in edges
         ]
 
@@ -265,7 +259,7 @@ class ObschartApiClient(object):
         variables = {"filter": filter}
         response = await self._execute(query, variables)
 
-        nodes: list[ProgramModule] = [
+        nodes: List[ProgramModule] = [
             ProgramModule(edge["node"], self._context)
             for edge in response["programModules"]["edges"]
         ]
@@ -291,7 +285,58 @@ class ObschartApiClient(object):
         variables = {"filter": filter}
         response = await self._execute(query, variables)
 
-        nodes: list[Tag] = [Tag(edge["node"], self._context) for edge in response["tags"]["edges"]]
+        nodes: List[Tag] = [Tag(edge["node"], self._context) for edge in response["tags"]["edges"]]
+
+        return nodes
+
+    async def list_chat_channels(self, filter: dict):
+        query = gql(
+            """
+          query ListChatChannels($filter: ChatChannelFilterInput) {
+            chatChannels(filter: $filter, first: 9999) {
+              edges {
+                node {
+                  ...ChatChannel
+                }
+              }
+            }
+          }
+        """
+            + chatChannelFragment
+        )
+
+        variables = {"filter": filter}
+        response = await self._execute(query, variables)
+
+        nodes: List[ChatChannel] = [
+            ChatChannel(edge["node"], self._context) for edge in response["chatChannels"]["edges"]
+        ]
+
+        return nodes
+
+    async def list_client_profile_invitations(self, filter: dict):
+        query = gql(
+            """
+          query ListClientProfileInvitations($filter: ClientProfileInvitationFilterInput) {
+            clientProfileInvitations(filter: $filter, first: 9999) {
+              edges {
+                node {
+                  ...ClientProfileInvitation
+                }
+              }
+            }
+          }
+        """
+            + clientProfileInvitationFragment
+        )
+
+        variables = {"filter": filter}
+        response = await self._execute(query, variables)
+
+        nodes: List[ClientProfileInvitation] = [
+            ClientProfileInvitation(edge["node"], self._context)
+            for edge in response["clientProfileInvitations"]["edges"]
+        ]
 
         return nodes
 
@@ -316,3 +361,21 @@ class ObschartApiClient(object):
         result = await self._context.client._execute_mutation(query, input)
 
         return ProgramModule(result["updateProgramModule"]["programModule"], self._execute,)
+
+    async def delete_node(self, id: str):
+        query = gql(
+            """
+            mutation DeleteNodeMutation($input: DeleteNodeInput!)  {
+                deleteNode(input: $input) {
+                    node {
+                        id
+                    }
+                }
+            }
+        """
+        )
+
+        input = {
+            "id": id,
+        }
+        await self._context.client._execute_mutation(query, input)
