@@ -4,10 +4,10 @@ from typing import Optional, Union, Any, Callable
 from .blocks_action_builder import BlocksActionBuilder
 from typing import Awaitable
 import asyncio
-import traceback
 from .gql import GqlClient
 from .api import ObschartApiClient
 import io
+import logging
 
 try:
     import matplotlib.pyplot as plt
@@ -16,6 +16,8 @@ except ModuleNotFoundError:
 
 OnResponseCallback = Callable[[ProgramTrackActionResponse], Awaitable[None]]
 
+log = logging.getLogger(__name__)
+
 
 class ObschartClient(ObschartApiClient):
     _transport: ObschartApiTransport
@@ -34,7 +36,7 @@ class ObschartClient(ObschartApiClient):
             self.set_authentication_token(authentication_token)
 
     def set_authentication_token(self, authentication_token: str):
-        self._transport._headers["Authorization"] = f"Bearer {authentication_token}"
+        self._transport._client.headers["Authorization"] = f"Bearer {authentication_token}"
 
     async def login(self, email: str, password: str):
         authentication_token, session = await self.create_session(email, password)
@@ -48,18 +50,22 @@ class ObschartClient(ObschartApiClient):
                     {"applicationId": {"eq": "me"}}
                 )
 
-                results = await asyncio.gather(
-                    *[on_response_callback(response) for response in responses],
-                    return_exceptions=True,
-                )
+                for response in responses:
 
-                for result in results:
-                    if isinstance(result, Exception):
-                        exception = result
-                        traceback.print_exception(None, exception, exception.__traceback__)
+                    async def task():
+                        try:
+                            log.debug("Handling response...")
+                            await on_response_callback(response)
+                            log.debug("Response handled")
+                        except:
+                            log.exception("Could not handle response: %s", response)
+
+                    asyncio.ensure_future(task())
 
         loop = asyncio.get_event_loop()
         loop.run_until_complete(main())
+        # https://stackoverflow.com/a/37345564/11225752
+        # loop.run_until_complete(asyncio.Task.all_tasks())
 
     def build_feedback_data(self):
         return BlocksActionBuilder()
